@@ -460,6 +460,9 @@ class MySQLCursor(CursorBase):
         """
         if not operation:
             return
+
+        if not self._connection:
+            raise errors.ProgrammingError("Cursor is not connected.")
         if self._have_unread_result():
             raise errors.InternalError("Unread result found.")
 
