@@ -18,8 +18,9 @@ Requires and comes with MySQL Connector/Python v1.1 and later:
 from __future__ import unicode_literals
 
 import sys
-import warnings
+
 import django
+from django.utils.functional import cached_property
 
 try:
     import mysql.connector
@@ -42,7 +43,12 @@ if version < (1, 1):
         "is required; you have %s" % mysql.connector.__version__)
 
 from django.db import utils
-from django.db.backends import *
+if django.VERSION < (1, 7):
+    from django.db.backends import util
+else:
+    from django.db.backends import utils as backend_utils
+from django.db.backends import (BaseDatabaseFeatures, BaseDatabaseOperations,
+                                BaseDatabaseWrapper)
 from django.db.backends.signals import connection_created
 from django.utils import (six, timezone, dateparse)
 from django.conf import settings
@@ -51,12 +57,14 @@ from mysql.connector.django.client import DatabaseClient
 from mysql.connector.django.creation import DatabaseCreation
 from mysql.connector.django.introspection import DatabaseIntrospection
 from mysql.connector.django.validation import DatabaseValidation
+if django.VERSION >= (1, 7):
+    from mysql.connector.django.schema import DatabaseSchemaEditor
 
-# Raise exceptions for database warnings if DEBUG is on
-# Note: PEP-249 says Warning must be subclass of StandardError
-mysql.connector.Warning = Warning
-if settings.DEBUG:
-    warnings.filterwarnings("error", category=mysql.connector.Warning)
+try:
+    import pytz
+    HAVE_PYTZ = True
+except ImportError:
+    HAVE_PYTZ = False
 
 DatabaseError = mysql.connector.DatabaseError
 IntegrityError = mysql.connector.IntegrityError
@@ -70,7 +78,7 @@ class DjangoMySQLConverter(MySQLConverter):
 
         Returns datetime.time()
         """
-        return dateparse.parse_time(value)
+        return dateparse.parse_time(value.decode('utf-8'))
 
     def _DATETIME_to_python(self, value, dsc=None):
         """Connector/Python always returns naive datetime.datetime
@@ -103,8 +111,10 @@ class CursorWrapper(object):
         """Wrapper around execute() and executemany()"""
         try:
             return method(query, args)
-        except (mysql.connector.ProgrammingError,
-                mysql.connector.IntegrityError) as err:
+        except (mysql.connector.ProgrammingError) as err:
+            six.reraise(utils.ProgrammingError,
+                        utils.ProgrammingError(err.msg), sys.exc_info()[2])
+        except (mysql.connector.IntegrityError) as err:
             six.reraise(utils.IntegrityError,
                         utils.IntegrityError(err.msg), sys.exc_info()[2])
         except mysql.connector.OperationalError as err:
@@ -128,7 +138,7 @@ class CursorWrapper(object):
         This wrapper method around the executemany()-method of the cursor is
         mainly needed to re-raise using different exceptions.
         """
-        return self._execute_wrapper(self.cursor.execute, query, args)
+        return self._execute_wrapper(self.cursor.executemany, query, args)
 
     def __getattr__(self, attr):
         """Return attribute of wrapped cursor"""
@@ -138,6 +148,12 @@ class CursorWrapper(object):
         """Returns iterator over wrapped cursor"""
         return iter(self.cursor)
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        self.close()
+
 
 class DatabaseFeatures(BaseDatabaseFeatures):
     """Features specific to MySQL
@@ -155,24 +171,30 @@ class DatabaseFeatures(BaseDatabaseFeatures):
     has_select_for_update_nowait = False
     supports_forward_references = False
     supports_long_model_names = False
+    supports_binary_field = six.PY2
     supports_microsecond_precision = False  # toggled in __init__()
     supports_regex_backreferencing = False
     supports_date_lookup_using_string = False
+    can_introspect_binary_field = False
+    can_introspect_boolean_field = False
     supports_timezones = False
     requires_explicit_null_ordering_when_grouping = True
+    allows_auto_pk_0 = False
     allows_primary_key_0 = False
     uses_savepoints = True
     atomic_transactions = False
+    supports_column_check_constraints = False
 
     def __init__(self, connection):
         super(DatabaseFeatures, self).__init__(connection)
         self.supports_microsecond_precision = self._microseconds_precision()
 
     def _microseconds_precision(self):
-        if self.connection.get_server_version() >= (5, 6, 3):
+        if self.connection.server_version >= (5, 6, 3):
             return True
         return False
 
+    @cached_property
     def _mysql_storage_engine(self):
         """Get default storage engine of MySQL
 
@@ -181,28 +203,30 @@ class DatabaseFeatures(BaseDatabaseFeatures):
 
         Used by Django tests.
         """
-        cursor = self.connection.cursor()
         tblname = 'INTROSPECT_TEST'
 
         droptable = 'DROP TABLE IF EXISTS {table}'.format(table=tblname)
+        with self.connection.cursor() as cursor:
+            cursor.execute(droptable)
+            cursor.execute('CREATE TABLE {table} (X INT)'.format(table=tblname))
 
-        cursor.execute(droptable)
-        cursor.execute('CREATE TABLE {table} (X INT)'.format(table=tblname))
+            if self.connection.server_version >= (5, 0, 0):
+                cursor.execute(
+                    "SELECT ENGINE FROM INFORMATION_SCHEMA.TABLES "
+                    "WHERE TABLE_SCHEMA = %s AND TABLE_NAME = %s",
+                    (self.connection.settings_dict['NAME'], tblname))
+                engine = cursor.fetchone()[0]
+            else:
+                # Very old MySQL servers..
+                cursor.execute("SHOW TABLE STATUS WHERE Name='{table}'".format(
+                    table=tblname))
+                engine = cursor.fetchone()[1]
+            cursor.execute(droptable)
 
-        if self.connection.get_server_version() >= (5, 0, 0):
-            cursor.execute(
-                "SELECT ENGINE FROM INFORMATION_SCHEMA.TABLES "
-                "WHERE TABLE_SCHEMA = %s AND TABLE_NAME = %s",
-                (self.connection.settings_dict['NAME'], tblname))
-            engine = cursor.fetchone()[0]
-        else:
-            # Very old MySQL servers..
-            cursor.execute("SHOW TABLE STATUS WHERE Name='{table}'".format(
-                table=tblname))
-            engine = cursor.fetchone()[1]
-        cursor.execute(droptable)
+        self._cached_storage_engine = engine
         return engine
 
+    @cached_property
     def can_introspect_foreign_keys(self):
         """Confirm support for introspected foreign keys
 
@@ -211,6 +235,7 @@ class DatabaseFeatures(BaseDatabaseFeatures):
         """
         return self._mysql_storage_engine == 'InnoDB'
 
+    @cached_property
     def has_zoneinfo_database(self):
         """Tests if the time zone definitions are installed
 
@@ -221,16 +246,24 @@ class DatabaseFeatures(BaseDatabaseFeatures):
         MySQL cannot perform time zone conversions reliably.
         """
         # Django 1.6
-        if pytz is None:
+        if not HAVE_PYTZ:
             return False
 
-        self.connection.cmd_query("SELECT 1 FROM mysql.time_zone LIMIT 1")
-        return self.connection.get_rows()[0] is not []
+        with self.connection.cursor() as cursor:
+            cursor.execute("SELECT 1 FROM mysql.time_zone LIMIT 1")
+            return cursor.fetchall() != []
 
 
 class DatabaseOperations(BaseDatabaseOperations):
     compiler_module = "mysql.connector.django.compiler"
 
+    # MySQL stores positive fields as UNSIGNED ints.
+    if django.VERSION >= (1, 7):
+        integer_field_ranges = dict(BaseDatabaseOperations.integer_field_ranges,
+                                    PositiveSmallIntegerField=(0, 4294967295),
+                                    PositiveIntegerField=(
+                                        0, 18446744073709551615),)
+
     def date_extract_sql(self, lookup_type, field_name):
         # http://dev.mysql.com/doc/mysql/en/date-and-time-functions.html
         if lookup_type == 'week_day':
@@ -250,7 +283,10 @@ class DatabaseOperations(BaseDatabaseOperations):
         The field_name is returned when lookup_type is not supported.
         """
         fields = ['year', 'month', 'day', 'hour', 'minute', 'second']
-        format = ('%%Y-', '%%m', '-%%d', ' %%H:', '%%i', ':%%s')
+        if sys.version_info[0] == 3:
+            format_ = ('%Y-', '%m', '-%d', ' %H:', '%i', ':%s')
+        else:
+            format_ = ('%%Y-', '%%m', '-%%d', ' %%H:', '%%i', ':%%s')
         format_def = ('0000-', '01', '-01', ' 00:', '00', ':00')
         try:
             i = fields.index(lookup_type) + 1
@@ -258,7 +294,7 @@ class DatabaseOperations(BaseDatabaseOperations):
             # Wrong lookup type, just return the value from MySQL as-is
             sql = field_name
         else:
-            format_str = ''.join([f for f in format[:i]] +
+            format_str = ''.join([f for f in format_[:i]] +
                                  [f for f in format_def[i:]])
             sql = "CAST(DATE_FORMAT({0}, '{1}') AS DATETIME)".format(
                 field_name, format_str)
@@ -290,7 +326,10 @@ class DatabaseOperations(BaseDatabaseOperations):
         else:
             params = []
         fields = ['year', 'month', 'day', 'hour', 'minute', 'second']
-        format_ = ('%%Y-', '%%m', '-%%d', ' %%H:', '%%i', ':%%s')
+        if sys.version_info[0] == 3:
+            format_ = ('%Y-', '%m', '-%d', ' %H:', '%i', ':%s')
+        else:
+            format_ = ('%%Y-', '%%m', '-%%d', ' %%H:', '%%i', ':%%s')
         format_def = ('0000-', '01', '-01', ' 00:', '00', ':00')
         try:
             i = fields.index(lookup_type) + 1
@@ -364,7 +403,7 @@ class DatabaseOperations(BaseDatabaseOperations):
         # Truncate already resets the AUTO_INCREMENT field from
         # MySQL version 5.0.13 onwards. Refs #16961.
         res = []
-        if self.connection.get_server_version() < (5, 0, 13):
+        if self.connection.server_version < (5, 0, 13):
             fmt = "{alter} {table} {{tablename}} {auto_inc} {field};".format(
                 alter=style.SQL_KEYWORD('ALTER'),
                 table=style.SQL_KEYWORD('TABLE'),
@@ -379,13 +418,13 @@ class DatabaseOperations(BaseDatabaseOperations):
 
     def validate_autopk_value(self, value):
         # MySQLism: zero in AUTO_INCREMENT field does not work. Refs #17653.
-        if not value:
+        if value == 0:
             raise ValueError('The database backend does not accept 0 as a '
                              'value for AutoField.')
         return value
 
     def value_to_db_datetime(self, value):
-        if not value:
+        if value is None:
             return None
         # MySQL doesn't support tz-aware times
         if timezone.is_aware(value):
@@ -395,10 +434,17 @@ class DatabaseOperations(BaseDatabaseOperations):
                 raise ValueError(
                     "MySQL backend does not support timezone-aware times."
                 )
+
+        try:
+            # Django 1.6
+            self.connection.ensure_connection()
+        except AttributeError:
+            if not self.connection.connection:
+                self.connection._connect()
         return self.connection.connection.converter._datetime_to_mysql(value)
 
     def value_to_db_time(self, value):
-        if not value:
+        if value is None:
             return None
 
         # MySQL doesn't support tz-aware times
@@ -406,6 +452,12 @@ class DatabaseOperations(BaseDatabaseOperations):
             raise ValueError("MySQL backend does not support timezone-aware "
                              "times.")
 
+        try:
+            # Django 1.6
+            self.connection.ensure_connection()
+        except AttributeError:
+            if not self.connection.connection:
+                self.connection._connect()
         return self.connection.connection.converter._time_to_mysql(value)
 
     def year_lookup_bounds(self, value):
@@ -419,7 +471,7 @@ class DatabaseOperations(BaseDatabaseOperations):
         # Again, no microseconds
         first, second = super(DatabaseOperations,
             self).year_lookup_bounds_for_datetime_field(value)
-        if self.connection.get_server_version() >= (5, 6, 4):
+        if self.connection.server_version >= (5, 6, 4):
             return [first.replace(microsecond=0), second]
         else:
             return [first.replace(microsecond=0),
@@ -441,6 +493,15 @@ class DatabaseOperations(BaseDatabaseOperations):
     def savepoint_rollback_sql(self, sid):
         return "ROLLBACK TO SAVEPOINT {0}".format(sid)
 
+    def combine_expression(self, connector, sub_expressions):
+        """
+        MySQL requires special cases for ^ operators in query expressions
+        """
+        if connector == '^':
+            return 'POW(%s)' % ','.join(sub_expressions)
+        return super(DatabaseOperations, self).combine_expression(
+            connector, sub_expressions)
+
 
 class DatabaseWrapper(BaseDatabaseWrapper):
     vendor = 'mysql'
@@ -468,10 +529,14 @@ class DatabaseWrapper(BaseDatabaseWrapper):
         self.server_version = None
 
         # Since some features depend on the MySQL version, we need to connect
-        self._connect()
+        try:
+            # Django 1.6
+            self.ensure_connection()
+        except AttributeError:
+            self._connect()
 
-        self.features = DatabaseFeatures(self)
         self.ops = DatabaseOperations(self)
+        self.features = DatabaseFeatures(self)
         self.client = DatabaseClient(self)
         self.creation = DatabaseCreation(self)
         self.introspection = DatabaseIntrospection(self)
@@ -487,7 +552,6 @@ class DatabaseWrapper(BaseDatabaseWrapper):
         kwargs = {
             'charset': 'utf8',
             'use_unicode': True,
-            'sql_mode': 'TRADITIONAL',
             'buffered': True,
         }
 
@@ -506,11 +570,18 @@ class DatabaseWrapper(BaseDatabaseWrapper):
         if settings_dict['PORT']:
             kwargs['port'] = int(settings_dict['PORT'])
 
+        # Raise exceptions for database warnings if DEBUG is on
+        kwargs['raise_on_warnings'] = settings.DEBUG
+
         kwargs['client_flags'] = [
             # Need potentially affected rows on UPDATE
             mysql.connector.constants.ClientFlag.FOUND_ROWS,
         ]
-        kwargs.update(settings_dict['OPTIONS'])
+        try:
+            kwargs.update(settings_dict['OPTIONS'])
+        except KeyError:
+            # OPTIONS missing is OK
+            pass
 
         return kwargs
 
@@ -528,8 +599,14 @@ class DatabaseWrapper(BaseDatabaseWrapper):
         if self.server_version < (5, 5, 3):
             # See sysvar_sql_auto_is_null in MySQL Reference manual
             self.connection.cmd_query("SET SQL_AUTO_IS_NULL = 0")
-        if self.settings_dict['AUTOCOMMIT']:
-            self.set_autocommit(self.settings_dict['AUTOCOMMIT'])
+
+        if 'AUTOCOMMIT' in self.settings_dict:
+            try:
+                # Django 1.6
+                self.set_autocommit(self.settings_dict['AUTOCOMMIT'])
+            except AttributeError:
+                self._set_autocommit(self.settings_dict['AUTOCOMMIT'])
+
 
     def create_cursor(self):
         # Django 1.6
@@ -559,6 +636,13 @@ class DatabaseWrapper(BaseDatabaseWrapper):
 
         Returns a tuple
         """
+        try:
+            # Django 1.6
+            self.ensure_connection()
+        except AttributeError:
+            if not self.connection:
+                self._connect()
+
         return self.server_version
 
     def disable_constraint_checking(self):
@@ -646,8 +730,18 @@ class DatabaseWrapper(BaseDatabaseWrapper):
 
     def _set_autocommit(self, autocommit):
         # Django 1.6
-        self.connection.autocommit = autocommit
+        with self.wrap_database_errors:
+            self.connection.autocommit = autocommit
+
+    def schema_editor(self, *args, **kwargs):
+        """Returns a new instance of this backend's SchemaEditor"""
+        # Django 1.7
+        return DatabaseSchemaEditor(self, *args, **kwargs)
 
     def is_usable(self):
         # Django 1.6
         return self.connection.is_connected()
+
+    @property
+    def mysql_version(self):
+        return self.server_version
