@@ -40,6 +40,7 @@ from mysql.connector import errors
 from mysql.connector.utils import int4store
 from mysql.connector.cursor import (CursorBase, MySQLCursor, MySQLCursorRaw,
     MySQLCursorBuffered, MySQLCursorBufferedRaw, MySQLCursorPrepared)
+from mysql.connector.authentication import get_auth_plugin
 
 DEFAULT_CONFIGURATION = {
     'database': None,
@@ -71,6 +72,7 @@ DEFAULT_CONFIGURATION = {
     'connect_timeout': None,
     'dsn': None,
     'force_ipv6': False,
+    'auth_plugin': None,
 }
 
 
@@ -113,6 +115,9 @@ class MySQLConnection(object):
 
         self._prepared_statements = None
 
+        self._ssl_active = False
+        self._auth_plugin = None
+
         if len(kwargs) > 0:
             self.connect(**kwargs)
 
@@ -147,44 +152,78 @@ class MySQLConnection(object):
                 "MySQL Version '{}' is not supported.".format(
                     handshake['server_version_original']))
 
+        if handshake['capabilities'] & ClientFlag.PLUGIN_AUTH:
+            self.set_client_flags([ClientFlag.PLUGIN_AUTH])
+
         self._handshake = handshake
         self._server_version = version
 
     def _do_auth(self, username=None, password=None, database=None,
                  client_flags=0, charset=33, ssl_options=None):
         """Authenticate with the MySQL server
+
+        Authentication happens in two parts. We first send a response to the
+        handshake. The MySQL server will then send either an AuthSwitchRequest
+        or an error packet.
+
+        Raises NotSupportedError when we get the old, insecure password
+        reply back. Raises any error coming from MySQL.
         """
+        self._ssl_active = False
         if client_flags & ClientFlag.SSL and ssl_options:
             packet = self._protocol.make_auth_ssl(charset=charset,
                                                   client_flags=client_flags)
             self._socket.send(packet)
             self._socket.switch_to_ssl(**ssl_options)
+            self._ssl_active = True
 
         packet = self._protocol.make_auth(
-            seed=self._handshake['scramble'],
+            handshake=self._handshake,
             username=username, password=password, database=database,
-            charset=charset, client_flags=client_flags)
+            charset=charset, client_flags=client_flags,
+            ssl_enabled=self._ssl_active,
+            auth_plugin=self._auth_plugin)
         self._socket.send(packet)
-        packet = self._socket.recv()
+        self._auth_switch_request(username, password)
 
-        if packet[4] == 254:
-            raise errors.NotSupportedError(
-                "Authentication with old (insecure) passwords "
-                "is not supported. For more information, lookup "
-                "Password Hashing in the latest MySQL manual")
-        elif packet[4] == 255:
-            raise errors.get_exception(packet)
-
-        try:
-            if (not (client_flags & ClientFlag.CONNECT_WITH_DB)
-                and database):
-                self.cmd_init_db(database)
-        except:
-            raise
+        if (not (client_flags & ClientFlag.CONNECT_WITH_DB)
+            and database):
+            self.cmd_init_db(database)
 
         return True
 
-    def config(self, *args, **kwargs):
+    def _auth_switch_request(self, username=None, password=None):
+        """Handle second part of authentication
+
+        Raises NotSupportedError when we get the old, insecure password
+        reply back. Raises any error coming from MySQL.
+        """
+        packet = self._socket.recv()
+        if packet[4] == 254 and len(packet) == 5:
+            raise errors.NotSupportedError(
+              "Authentication with old (insecure) passwords "
+              "is not supported. For more information, lookup "
+              "Password Hashing in the latest MySQL manual")
+        elif packet[4] == 254:
+            # AuthSwitchRequest
+            (new_auth_plugin,
+             auth_data) = self._protocol.parse_auth_switch_request(packet)
+            auth = get_auth_plugin(new_auth_plugin)(
+                auth_data, password=password, ssl_enabled=self._ssl_active)
+            response = auth.auth_response()
+            if response == b'\x00':
+                self._socket.send(b'')
+            else:
+                self._socket.send(response)
+            packet = self._socket.recv()
+            if packet[4] != 1:
+                return self._handle_ok(packet)
+            else:
+                auth_data = self._protocol.parse_auth_more_data(packet)
+        elif packet[4] == 255:
+            raise errors.get_exception(packet)
+
+    def config(self, **kwargs):
         """Configure the MySQL Connection
 
         This method allows you to configure the MySQLConnection instance.
@@ -222,7 +261,7 @@ class MySQLConnection(object):
             pass  # Missing compress argument is OK
 
         # Configure character set and collation
-        if ('charset' in config or 'collation' in config):
+        if 'charset' in config or 'collation' in config:
             try:
                 charset = config['charset']
                 del config['charset']
@@ -261,7 +300,7 @@ class MySQLConnection(object):
                 pass  # Missing compat argument is OK
 
         # Configure login information
-        if ('user' in config or 'password' in config):
+        if 'user' in config or 'password' in config:
             try:
                 user = config['user']
                 del config['user']
@@ -292,10 +331,7 @@ class MySQLConnection(object):
             except KeyError:
                 raise AttributeError("Unsupported argument '{}'".format(key))
             # SSL Configuration
-            if key == 'ssl_verify_cert':
-                set_ssl_flag = True
-                self._ssl.update({'verify_cert': value})
-            elif key.startswith('ssl_') and value:
+            if key.startswith('ssl_'):
                 set_ssl_flag = True
                 self._ssl.update({key.replace('ssl_', ''): value})
             else:
@@ -309,12 +345,24 @@ class MySQLConnection(object):
             if 'verify_cert' not in self._ssl:
                 self._ssl['verify_cert'] = \
                     DEFAULT_CONFIGURATION['ssl_verify_cert']
-            required_keys = set(['ca', 'cert', 'key'])
-            diff = list(required_keys - set(self._ssl.keys()))
-            if diff:
-                missing_attrs = ["ssl_" + val for val in diff]
-                raise AttributeError("Missing SSL argument(s): {0}".format(
-                    ', '.join(missing_attrs)))
+            # Make sure both ssl_key/ssl_cert are set, or neither (XOR)
+            if 'ca' not in self._ssl or self._ssl['ca'] is None:
+                raise AttributeError(
+                    "Missing ssl_ca argument.")
+            if bool('key' in self._ssl) != bool('cert' in self._ssl):
+                raise AttributeError(
+                    "ssl_key and ssl_cert need to be both "
+                    "specified, or neither."
+                )
+            # Make sure key/cert are set to None
+            elif not set(('key', 'cert')) <= set(self._ssl):
+                self._ssl['key'] = None
+                self._ssl['cert'] = None
+            elif (self._ssl['key'] is None) != (self._ssl['cert'] is None):
+                raise AttributeError(
+                    "ssl_key and ssl_cert need to be both "
+                    "set, or neither."
+                )
             self.set_client_flags([ClientFlag.SSL])
 
     def _get_connection(self, prtcls=None):
@@ -800,11 +848,26 @@ class MySQLConnection(object):
             raise errors.InternalError("Unread result found.")
 
         packet = self._protocol.make_change_user(
-            seed=self._handshake['scramble'],
+            handshake=self._handshake,
             username=username, password=password, database=database,
-            charset=charset, client_flags=self._client_flags)
+            charset=charset, client_flags=self._client_flags,
+            ssl_enabled=self._ssl_active,
+            auth_plugin=self._auth_plugin)
         self._socket.send(packet, 0)
-        return self._handle_ok(self._socket.recv())
+
+        ok_packet = self._auth_switch_request(username, password)
+
+        try:
+            if (not (self._client_flags & ClientFlag.CONNECT_WITH_DB)
+                and database):
+                self.cmd_init_db(database)
+        except:
+            raise
+
+        self._charset_id = charset
+        self._post_connection()
+
+        return ok_packet
 
     def is_connected(self):
         """Reports whether the connection to MySQL Server is available
@@ -821,6 +884,39 @@ class MySQLConnection(object):
             return False  # This method does not raise
         return True
 
+    def reset_session(self, user_variables=None, session_variables=None):
+        """Clears the current active session
+
+        This method resets the session state, if the MySQL server is 5.7.3
+        or later active session will be reset without re-authenticating.
+        For other server versions session will be reset by re-authenticating.
+
+        It is possible to provide a sequence of variables and their values to
+        be set after clearing the session. This is possible for both user
+        defined variables and session variables.
+        This method takes two arguments user_variables and session_variables
+        which are dictionaries.
+
+        Raises OperationalError if not connected, InternalError if there are
+        unread results and InterfaceError on errors.
+        """
+        if not self.is_connected():
+            raise errors.OperationalError("MySQL Connection not available.")
+
+        try:
+            self.cmd_reset_connection()
+        except errors.NotSupportedError:
+            self.cmd_change_user(self._user, self._password,
+                                 self._database, self._charset_id)
+
+        cur = self.cursor()
+        if user_variables:
+            for key, value in user_variables.items():
+                cur.execute("SET @`{0}` = %s".format(key), (value,))
+        if session_variables:
+            for key, value in session_variables.items():
+                cur.execute("SET SESSION `{0}` = %s".format(key), (value,))
+
     def reconnect(self, attempts=1, delay=0):
         """Attempt to reconnect to the MySQL server
 
@@ -1146,6 +1242,7 @@ class MySQLConnection(object):
         """Toggle autocommit"""
         switch = 'ON' if value else 'OFF'
         self._execute_query("SET @@session.autocommit = {}".format(switch))
+        self._autocommit = value
 
     def get_autocommit(self):
         """Get whether autocommit is on or off"""
@@ -1433,7 +1530,8 @@ class MySQLConnection(object):
                     long_data_used[param_id] = (binary,)
 
         execute_packet = self._protocol.make_stmt_execute(
-            statement_id, data, tuple(parameters), flags, long_data_used)
+            statement_id, data, tuple(parameters), flags,
+            long_data_used, self.charset)
         packet = self._send_cmd(ServerCmd.STMT_EXECUTE, packet=execute_packet)
         result = self._handle_binary_result(packet)
         return result
@@ -1492,3 +1590,18 @@ class MySQLConnection(object):
         """
         self._handle_ok(self._send_cmd(ServerCmd.STMT_RESET,
                                        int4store(statement_id)))
+
+    def cmd_reset_connection(self):
+        """Resets the session state without re-authenticating
+
+        Works only for MySQL server 5.7.3 or later.
+        The result is a dictionary with OK packet information.
+
+        Returns a dict()
+        """
+        if self._server_version < (5, 7, 3):
+            raise errors.NotSupportedError("MySQL version 5.7.2 and "
+                                           "earlier does not support "
+                                           "COM_RESET_CONNECTION.")
+        self._handle_ok(self._send_cmd(ServerCmd.RESET_CONNECTION))
+        self._post_connection()
