@@ -1,5 +1,5 @@
 # MySQL Connector/Python - MySQL driver written in Python.
-# Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
 
 # MySQL Connector/Python is licensed under the terms of the GPLv2
 # <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -287,6 +287,8 @@ class ServerCmd(_Constants):
     SET_OPTION      = 27
     STMT_FETCH      = 28
     DAEMON          = 29
+    BINLOG_DUMP_GTID = 30
+    RESET_CONNECTION = 31
 
     desc = {
         'SLEEP': (0, 'SLEEP'),
@@ -319,6 +321,8 @@ class ServerCmd(_Constants):
         'SET_OPTION': (27, 'SET_OPTION'),
         'STMT_FETCH': (28, 'STMT_FETCH'),
         'DAEMON': (29, 'DAEMON'),
+        'BINLOG_DUMP_GTID': (30, 'BINLOG_DUMP_GTID'),
+        'RESET_CONNECTION': (31, 'RESET_CONNECTION'),
     }
 
 
@@ -345,6 +349,11 @@ class ClientFlag(_Flags):
     SECURE_CONNECTION       = 1 << 15
     MULTI_STATEMENTS        = 1 << 16
     MULTI_RESULTS           = 1 << 17
+    PS_MULTI_RESULTS = 1 << 18
+    PLUGIN_AUTH = 1 << 19
+    CONNECT_ARGS = 1 << 20
+    PLUGIN_AUTH_LENENC_CLIENT_DATA = 1 << 21
+    CAN_HANDLE_EXPIRED_PASSWORDS = 1 << 22
     SSL_VERIFY_SERVER_CERT  = 1 << 30
     REMEMBER_OPTIONS        = 1 << 31
 
