@@ -264,14 +264,27 @@ class MySQLConverter(MySQLConverterBase):
 
         Returns a bytes.
         """
-        (hours, remainder) = divmod(value.seconds, 3600)
-        (mins, secs) = divmod(remainder, 60)
-        hours = hours + (value.days * 24)
+        seconds = abs(value.days * 86400 + value.seconds)
+
         if value.microseconds:
-            return '{:02d}:{:02d}:{:02d}.{:06d}'.format(
-                hours, mins, secs, value.microseconds).encode('ascii')
-        return '{:02d}:{:02d}:{:02d}'.format(
-            hours, mins, secs).encode('ascii')
+            fmt = '{0:02d}:{1:02d}:{2:02d}.{3:06d}'
+            if value.days < 0:
+                mcs = 1000000 - value.microseconds
+                seconds -= 1
+            else:
+                mcs = value.microseconds
+        else:
+            fmt = '{0:02d}:{1:02d}:{2:02d}'
+
+        if value.days < 0:
+            fmt = '-' + fmt
+
+        (hours, remainder) = divmod(seconds, 3600)
+        (mins, secs) = divmod(remainder, 60)
+
+        if value.microseconds:
+            return fmt.format(hours, mins, secs, mcs).encode('ascii')
+        return fmt.format(hours, mins, secs).encode('ascii')
 
     def _decimal_to_mysql(self, value):
         """
@@ -313,7 +326,10 @@ class MySQLConverter(MySQLConverterBase):
             return self._cache_field_types[flddsc[1]](value, flddsc)
         except KeyError:
             # If one type is not defined, we just return the value as str
-            return value.decode('utf-8')
+            try:
+                return value.decode('utf-8')
+            except UnicodeDecodeError:
+                return value
         except ValueError as err:
             raise ValueError("%s (field %s)" % (err, flddsc[0]))
         except TypeError as err:
@@ -383,9 +399,11 @@ class MySQLConverter(MySQLConverterBase):
             hms = value
             mcs = 0
         try:
-            (hrs, mins, sec) = [int(i) for i in hms.split(b':')]
-            time_val = datetime.timedelta(hours=hrs, minutes=mins, seconds=sec,
-                                    microseconds=mcs)
+            (hours, mins, secs) = [int(d) for d in hms.split(b':')]
+            if value[0] == 45:
+                mins, secs, mcs = -mins, -secs, -mcs
+            time_val = datetime.timedelta(hours=hours, minutes=mins,
+                                          seconds=secs, microseconds=mcs)
         except ValueError:
             raise ValueError(
                 "Could not convert {} to python datetime.timedelta".format(
