@@ -1,5 +1,5 @@
 # MySQL Connector/Python - MySQL driver written in Python.
-# Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
 
 # MySQL Connector/Python is licensed under the terms of the GPLv2
 # <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -100,9 +100,8 @@ def process_terminate(pid):
     """
     if os.name == 'nt':
         winkernel = ctypes.windll.kernel32
-        process = winkernel.OpenProcess(1, 0, pid)
-        winkernel.TerminateProcess(process, -1)
-        winkernel.WaitForSingleObject(process)
+        process = winkernel.OpenProcess(0x0001, 0, pid)  # PROCESS_TERMINATE
+        winkernel.TerminateProcess(process, 1)
         winkernel.CloseHandle(process)
     else:
         os.kill(pid, signal.SIGTERM)
@@ -173,8 +172,7 @@ class MySQLServerBase(object):
 
         if not self._sbindir:
             raise MySQLBootstrapError(
-                "MySQL binaries not found under {0}. Use --with-mysql".format(
-                    self._basedir))
+                "MySQL binaries not found under {0}".format(self._basedir))
 
         # Try to locate errmsg.sys and mysql_system_tables.sql
         if not self._sharedir:
@@ -192,7 +190,9 @@ class MySQLServerBase(object):
                 break
             for afile in files:
                 if afile == 'errmsg.sys' and 'english' in root:
-                    self._lc_messages_dir = self._sharedir
+                    self._lc_messages_dir = os.path.abspath(
+                        os.path.join(root, os.pardir)
+                    )
                 elif afile == 'mysql_system_tables.sql':
                     self._scriptdir = root
 
@@ -363,8 +363,6 @@ class MySQLServer(MySQLServerBase):
             '--basedir=%s' % self._basedir,
             '--datadir=%s' % self._datadir,
             '--log-warnings=0',
-            #'--loose-skip-innodb',
-            '--loose-skip-ndbcluster',
             '--max_allowed_packet=8M',
             '--default-storage-engine=myisam',
             '--net_buffer_length=16K',
@@ -377,6 +375,9 @@ class MySQLServer(MySQLServerBase):
                 '--lc-messages-dir={0}'.format(self._lc_messages_dir),
                 '--lc-messages=en_US'
             ])
+        if self._version[0:2] >= (5, 1):
+            cmd.append('--loose-skip-ndbcluster')
+
         return cmd
 
     def bootstrap(self):
@@ -404,6 +405,28 @@ class MySQLServer(MySQLServerBase):
             "CREATE DATABASE myconnpy;"
         ]
 
+        if self._version[0:3] >= (5, 7, 4):
+            # MySQL 5.7.4 only creates root@localhost
+            extra_sql.append(
+                "INSERT INTO mysql.user SELECT '127.0.0.1', `User`, `Password`,"
+                " `Select_priv`, `Insert_priv`, `Update_priv`, `Delete_priv`,"
+                " `Create_priv`, `Drop_priv`, `Reload_priv`, `Shutdown_priv`,"
+                " `Process_priv`, `File_priv`, `Grant_priv`, `References_priv`,"
+                " `Index_priv`, `Alter_priv`, `Show_db_priv`, `Super_priv`,"
+                " `Create_tmp_table_priv`, `Lock_tables_priv`, `Execute_priv`,"
+                " `Repl_slave_priv`, `Repl_client_priv`, `Create_view_priv`,"
+                " `Show_view_priv`, `Create_routine_priv`, "
+                "`Alter_routine_priv`,"
+                " `Create_user_priv`, `Event_priv`, `Trigger_priv`, "
+                "`Create_tablespace_priv`, `ssl_type`, `ssl_cipher`,"
+                "`x509_issuer`, `x509_subject`, `max_questions`, `max_updates`,"
+                "`max_connections`, `max_user_connections`, `plugin`,"
+                "`authentication_string`, `password_expired`,"
+                "`password_last_changed`, `password_lifetime` FROM mysql.user "
+                "WHERE `user` = 'root' and `host` = 'localhost';"
+            )
+
+        bootstrap_log = os.path.join(self._topdir, 'bootstrap.log')
         try:
             self._create_directories()
             cmd = self._get_bootstrap_cmd()
@@ -414,19 +437,28 @@ class MySQLServer(MySQLServerBase):
                 with open(full_path, 'r') as fp:
                     sql.extend([line.strip() for line in fp.readlines()])
             sql.extend(extra_sql)
-            logfile = open(os.path.join(self._topdir, 'bootstrap.log'), 'w')
+            fp_log = open(bootstrap_log, 'w')
             prc = subprocess.Popen(cmd, stdin=subprocess.PIPE,
-                                   stderr=subprocess.STDOUT, stdout=logfile)
+                                   stderr=subprocess.STDOUT, stdout=fp_log)
             if sys.version_info[0] == 2:
                 prc.communicate('\n'.join(sql))
             else:
                 prc.communicate(bytearray('\n'.join(sql), 'utf8'))
-            logfile.close()
+            fp_log.close()
         except OSError as err:
             raise MySQLBootstrapError(
                 "Error bootstrapping MySQL '{name}': {error}".format(
                     name=self._name, error=str(err)))
 
+        with open(bootstrap_log, 'r') as fp:
+            log_lines = fp.readlines()
+            for log_line in log_lines:
+                if '[ERROR]' in log_line:
+                    err_msg = log_line.split('[ERROR]')[1].strip()
+                    raise MySQLBootstrapError(
+                        "Error bootstrapping MySQL '{name}': {error}".format(
+                            name=self._name, error=err_msg))
+
     @property
     def name(self):
         """Returns the name of this MySQL server"""
@@ -473,7 +505,7 @@ class MySQLServer(MySQLServerBase):
             fp.write(self._cnf.format(**options))
             fp.close()
             self._start_server()
-            time.sleep(3)
+            time.sleep(5)
         except MySQLServerError as err:
             if self._debug is True:
                 raise
