@@ -172,7 +172,7 @@ class MySQLCursorTests(tests.TestsCursor):
             'NULL',
             '128',
             '1281288',
-            '3.14',
+            repr(float(3.14)),
             "'3.14'",
             "'back\\\\slash'",
             "'newline\\n'",
@@ -231,7 +231,7 @@ class MySQLCursorTests(tests.TestsCursor):
             'a': 'NULL',
             'b': '128',
             'c': '1281288',
-            'd': '3.14',
+            'd': repr(float(3.14)),
             'e': "'3.14'",
             'f': "'back\\\\slash'",
             'g': "'newline\\n'",
@@ -511,18 +511,26 @@ class MySQLCursorTests(tests.TestsCursor):
             "OUT pConCat VARCHAR(100)) "
             "BEGIN SET pConCat := CONCAT(pStr1, pStr2); END;")
 
+        stmt_create4 = (
+            "CREATE PROCEDURE myconnpy_sp_4"
+            "(IN pStr1 VARCHAR(20), INOUT pStr2 VARCHAR(20), "
+            "OUT pConCat VARCHAR(100)) "
+            "BEGIN SET pConCat := CONCAT(pStr1, pStr2); END;")
+
         try:
             cursor = connection.cursor()
             cursor.execute(stmt_create1)
             cursor.execute(stmt_create2)
             cursor.execute(stmt_create3)
+            cursor.execute(stmt_create4)
         except errors.Error as err:
             self.fail("Failed setting up test stored routine; {0}".format(err))
         cursor.close()
 
     def _test_callproc_cleanup(self, connection):
 
-        sp_names = ('myconnpy_sp_1', 'myconnpy_sp_2', 'myconnpy_sp_3')
+        sp_names = ('myconnpy_sp_1', 'myconnpy_sp_2', 'myconnpy_sp_3',
+                    'myconnpy_sp_4')
         stmt_drop = "DROP PROCEDURE IF EXISTS {procname}"
 
         try:
@@ -547,12 +555,18 @@ class MySQLCursorTests(tests.TestsCursor):
         self._test_callproc_setup(self.connection)
         self.cur = self.connection.cursor()
 
-        exp = (5, 4, 20)
+        if tests.MYSQL_VERSION < (5, 1):
+            exp = ('5', '4', '20')
+        else:
+            exp = (5, 4, 20)
         result = self.cur.callproc('myconnpy_sp_1', (exp[0], exp[1], 0))
         self.assertEqual([], self.cur._stored_results)
         self.assertEqual(exp, result)
 
-        exp = (6, 5, 30)
+        if tests.MYSQL_VERSION < (5, 1):
+            exp = ('6', '5', '30')
+        else:
+            exp = (6, 5, 30)
         result = self.cur.callproc('myconnpy_sp_2', (exp[0], exp[1], 0))
         self.assertTrue(isinstance(self.cur._stored_results, list))
         self.assertEqual(exp, result)
@@ -570,6 +584,12 @@ class MySQLCursorTests(tests.TestsCursor):
         self.assertTrue(isinstance(self.cur._stored_results, list))
         self.assertEqual(exp, result)
 
+        exp = ('ham', 'spam', 'hamspam')
+        result = self.cur.callproc('myconnpy_sp_4',
+                              (exp[0], (exp[1], 'CHAR'), (0, 'CHAR')))
+        self.assertTrue(isinstance(self.cur._stored_results, list))
+        self.assertEqual(exp, result)
+
         self._test_callproc_cleanup(self.connection)
         self.cur.close()
 
