@@ -1,5 +1,5 @@
 # MySQL Connector/Python - MySQL driver written in Python.
-# Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
 
 # MySQL Connector/Python is licensed under the terms of the GPLv2
 # <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -60,15 +60,6 @@ class MySQLProtocolTests(tests.MySQLConnectorTests):
     def setUp(self):
         self._protocol = protocol.MySQLProtocol()
 
-    def test__scramble_password(self):
-        """Scramble a password ready to send to MySQL"""
-        password = 'spam'
-        hashed = '\x3a\x07\x66\xba\xba\x01\xce\xbe\x55\xe6'\
-                 '\x29\x88\xaa\xae\xdb\x00\xb3\x4d\x91\x5b'
-
-        res = self._protocol._scramble_password(password, SEED)
-        self.assertEqual(hashed, res)
-
     def test_make_auth(self):
         """Make a MySQL authentication packet"""
         exp = {
@@ -101,7 +92,7 @@ class MySQLProtocolTests(tests.MySQLConnectorTests):
         }
         flags = ClientFlag.get_default()
         kwargs = {
-            'seed': None,
+            'handshake': None,
             'username': 'ham',
             'password': 'spam',
             'database': 'test',
@@ -111,7 +102,14 @@ class MySQLProtocolTests(tests.MySQLConnectorTests):
         self.assertRaises(errors.ProgrammingError,
                           self._protocol.make_auth, **kwargs)
 
-        kwargs['seed'] = SEED
+        kwargs['handshake'] = {'auth_data': SEED}
+        self.assertRaises(errors.ProgrammingError,
+                          self._protocol.make_auth, **kwargs)
+
+        kwargs['handshake'] = {
+            'auth_data': SEED,
+            'auth_plugin': 'mysql_native_password'
+        }
         res = self._protocol.make_auth(**kwargs)
         self.assertEqual(exp['allset'], res)
 
@@ -178,7 +176,7 @@ class MySQLProtocolTests(tests.MySQLConnectorTests):
             '\x73\x74\x00\x08\x00',
         }
         kwargs = {
-            'seed': None,
+            'handshake': None,
             'username': 'ham',
             'password': 'spam',
             'database': 'test',
@@ -188,7 +186,14 @@ class MySQLProtocolTests(tests.MySQLConnectorTests):
         self.assertRaises(errors.ProgrammingError,
                           self._protocol.make_change_user, **kwargs)
 
-        kwargs['seed'] = SEED
+        kwargs['handshake'] = {'auth_data': SEED}
+        self.assertRaises(errors.ProgrammingError,
+                          self._protocol.make_change_user, **kwargs)
+
+        kwargs['handshake'] = {
+            'auth_data': SEED,
+            'auth_plugin': 'mysql_native_password'
+        }
         res = self._protocol.make_change_user(**kwargs)
         self.assertEqual(exp['allset'], res)
 
@@ -213,7 +218,8 @@ class MySQLProtocolTests(tests.MySQLConnectorTests):
             'server_threadid': 265,
             'capabilities': 41516,
             'server_status': 2,
-            'scramble': 'h4i6oP!OLng9&PD@WrYH'
+            'auth_data': 'h4i6oP!OLng9&PD@WrYH',
+            'auth_plugin': 'mysql_native_password',
         }
 
         res = self._protocol.parse_handshake(handshake)
