@@ -39,7 +39,7 @@ import os
 import sys
 import gc
 import tempfile
-from datetime import datetime
+from datetime import datetime, timedelta
 from threading import Thread
 import time
 import unittest
@@ -284,21 +284,24 @@ class Bug501290(tests.MySQLConnectorTests):
     def setUp(self):
         config = tests.get_mysql_config()
         self.cnx = connection.MySQLConnection(**config)
+        self.capabilities = self.cnx._handshake['capabilities']
+
+        self.default_flags = constants.ClientFlag.get_default()
+        if self.capabilities & constants.ClientFlag.PLUGIN_AUTH:
+            self.default_flags |= constants.ClientFlag.PLUGIN_AUTH
 
     def tearDown(self):
         self.cnx.close()
 
     def test_default(self):
         """lp:501290 Check default client flags"""
-        self.assertEqual(self.cnx._client_flags,
-                         constants.ClientFlag.get_default())
+        self.assertEqual(self.cnx._client_flags, self.default_flags)
 
     def test_set_unset(self):
         """lp:501290 Set/unset one flag, check if set/unset"""
         orig = self.cnx._client_flags
 
-        exp = constants.ClientFlag.get_default() | \
-            constants.ClientFlag.COMPRESS
+        exp = self.default_flags | constants.ClientFlag.COMPRESS
         self.cnx.set_client_flags([constants.ClientFlag.COMPRESS])
         self.assertEqual(self.cnx._client_flags, exp)
 
@@ -307,12 +310,11 @@ class Bug501290(tests.MySQLConnectorTests):
 
     def test_isset_client_flag(self):
         """lp:501290 Check if client flag is set"""
-        data = constants.ClientFlag.get_default() | \
-            constants.ClientFlag.COMPRESS
+        flag = constants.ClientFlag.COMPRESS
+        data = self.default_flags | flag
 
         self.cnx._client_flags = data
-        self.assertEqual(True,
-                         self.cnx.isset_client_flag(constants.ClientFlag.COMPRESS))
+        self.assertEqual(True, self.cnx.isset_client_flag(flag))
 
 
 class Bug507466(tests.MySQLConnectorTests):
@@ -1281,10 +1283,6 @@ class BugOra16316049(tests.MySQLConnectorTests):
             'ssl_key': None,
         })
 
-        # User requires SSL, but no values for ssl argument given
-        self.assertRaises(errors.ProgrammingError,
-                          connection.MySQLConnection, **config)
-
         # Use wrong value for ssl_ca
         config['ssl_ca'] = os.path.abspath(
             os.path.join(tests.SSL_DIR, 'tests_casdfasdfdsaa_cert.pem'))
@@ -1662,7 +1660,8 @@ class BugOra17022399(tests.MySQLConnectorTests):
             cur.execute("SELECT 1")
         except mysql.connector.OperationalError as err:
             self.assertEqual(2055, err.errno)
-            self.assertTrue('9 ' in str(err))
+            # 10038 is a Windows socket error
+            self.assertTrue('9 ' in str(err) or '10038' in str(err))
 
     def test_execute_compressed(self):
         config = self.config.copy()
@@ -1674,7 +1673,8 @@ class BugOra17022399(tests.MySQLConnectorTests):
             cur.execute("SELECT 1")
         except mysql.connector.OperationalError as err:
             self.assertEqual(2055, err.errno)
-            self.assertTrue('9 ' in str(err))
+            # 10038 is a Windows socket error
+            self.assertTrue('9 ' in str(err) or '10038' in str(err))
 
 
 class BugOra16369511(tests.MySQLConnectorTests):
@@ -2087,3 +2087,537 @@ class BugOra18040042(tests.MySQLConnectorTests):
             self.assertEqual(('2',), pcnx.get_rows()[0][0])
         else:
             self.assertEqual((b'2',), pcnx.get_rows()[0][0])
+
+
+class BugOra17965619(tests.MySQLConnectorTests):
+    """BUG#17965619: CALLPROC FUNCTION WITH BYTES PARAMETERS
+    """
+    def setUp(self):
+        self.cnx = connection.MySQLConnection(**tests.get_mysql_config())
+        procedure = ("DROP PROCEDURE IF EXISTS `proce_with_binary`")
+        self.cnx.cmd_query(procedure)
+
+        procedure = ("CREATE PROCEDURE `proce_with_binary` "
+                     "(data VARBINARY(512)) BEGIN END;")
+        self.cnx.cmd_query(procedure)
+
+    def tearDown(self):
+        procedure = ("DROP PROCEDURE IF EXISTS `proce_with_binary`")
+        self.cnx.cmd_query(procedure)
+        self.cnx.close()
+
+    def test_callproc(self):
+        cur = self.cnx.cursor()
+
+        output = cur.callproc('proce_with_binary',
+                              ((BugOra17965619Extra.data, 'BINARY'),))
+        self.assertEqual((BugOra17965619Extra.data,), output)
+
+        cur.close()
+
+
+class BugOra17054848(tests.MySQLConnectorTests):
+    """BUG#17054848: USE OF SSL SHOULD NOT REQUIRE SSL_CERT AND SSL_KEY
+    """
+    def setUp(self):
+        config = tests.get_mysql_config()
+        self.admin_cnx = connection.MySQLConnection(**config)
+
+        self.admin_cnx.cmd_query(
+            "GRANT ALL ON %s.* TO 'ssluser'@'%s' REQUIRE SSL" % (
+                config['database'], config['host']))
+
+    def tearDown(self):
+        config = tests.get_mysql_config()
+        self.admin_cnx.cmd_query("DROP USER 'ssluser'@'%s'" % (
+                                 config['host']))
+
+    def test_ssl(self):
+        if not tests.SSL_AVAILABLE:
+            tests.MESSAGES['WARNINGS'].append(
+                "BugOra16217667 test failed. Python lacks SSL support.")
+            return
+
+        ssl_ca = os.path.abspath(
+            os.path.join(tests.SSL_DIR, 'tests_CA_cert.pem'))
+        ssl_key = os.path.abspath(
+            os.path.join(tests.SSL_DIR, 'tests_client_key.pem'))
+
+        config = tests.get_mysql_config()
+        config['user'] = 'ssluser'
+        config['password'] = ''
+        config['unix_socket'] = None
+        config['ssl_verify_cert'] = False
+        config.update({
+            'ssl_ca': ssl_ca,
+        })
+
+        try:
+            cnx = connection.MySQLConnection(**config)
+        except errors.ProgrammingError:
+            self.fail("Failed authentication with SSL")
+
+        cnx.cmd_query("SHOW STATUS LIKE 'Ssl_cipher'")
+        res = cnx.get_rows()[0][0]
+        self.assertTrue(res != '')
+
+
+class BugOra16217765(tests.MySQLConnectorTests):
+
+    """BUG#16217765: Fix authentication plugin support
+    """
+
+    users = {
+        'sha256_password': {
+            'username': 'sha256user',
+            'password': 'sha256P@ss',
+        },
+        'mysql_native_password': {
+            'username': 'nativeuser',
+            'password': 'nativeP@ss',
+        },
+    }
+
+    def _create_user(self, cnx, user, password, host, database,
+                     plugin):
+
+        self._drop_user(cnx, user, host)
+        create_user = ("CREATE USER '{user}'@'{host}' "
+                       "IDENTIFIED WITH {plugin}")
+        cnx.cmd_query(create_user.format(user=user, host=host, plugin=plugin))
+
+        if plugin == 'sha256_password':
+            cnx.cmd_query("SET old_passwords = 2")
+        else:
+            cnx.cmd_query("SET old_passwords = 0")
+
+        passwd = ("SET PASSWORD FOR '{user}'@'{host}' = "
+                  "PASSWORD('{password}')").format(user=user, host=host,
+                                                   password=password)
+        cnx.cmd_query(passwd)
+
+        grant = "GRANT ALL ON {database}.* TO '{user}'@'{host}'"
+        cnx.cmd_query(grant.format(database=database, user=user, host=host))
+
+    def _drop_user(self, cnx, user, host):
+        try:
+            self.admin_cnx.cmd_query("DROP USER '{user}'@'{host}'".format(
+                host=host,
+                user=user))
+        except errors.DatabaseError:
+            # It's OK when drop fails
+            pass
+
+    def setUp(self):
+        config = tests.get_mysql_config()
+        self.host = config['host']
+        self.admin_cnx = connection.MySQLConnection(**config)
+
+    def tearDown(self):
+        return
+        for plugin_name, info in self.users.items():
+            self._drop_user(self.admin_cnx, info['username'], self.host)
+
+    @unittest.skipIf(tests.MYSQL_VERSION < (5, 6, 6),
+                     "MySQL {0} does not support sha256_password auth".format(
+                        tests.MYSQL_VERSION_TXT))
+    def test_sha256(self):
+        config = tests.get_mysql_config()
+        config['unix_socket'] = None
+        config.update({
+            'ssl_ca': tests.SSL_CA,
+            'ssl_cert': tests.SSL_CERT,
+            'ssl_key': tests.SSL_KEY,
+        })
+
+        auth_plugin = 'sha256_password'
+        user = self.users[auth_plugin]
+        self._create_user(self.admin_cnx, user['username'],
+                          user['password'],
+                          self.host,
+                          config['database'],
+                          plugin=auth_plugin)
+
+        config['user'] = user['username']
+        config['password'] = user['password']
+        config['client_flags'] = [constants.ClientFlag.PLUGIN_AUTH]
+
+        try:
+            cnx = connection.MySQLConnection(**config)
+        except:
+            self.fail("Connecting using sha256_password auth failed")
+
+        try:
+            cnx.cmd_change_user(config['user'], config['password'])
+        except:
+            self.fail("Changing user using sha256_password auth failed")
+
+    @unittest.skipIf(tests.MYSQL_VERSION < (5, 6, 6),
+                     "MySQL {0} does not support sha256_password auth".format(
+                        tests.MYSQL_VERSION_TXT))
+    def test_sha256_nonssl(self):
+        config = tests.get_mysql_config()
+        config['unix_socket'] = None
+
+        auth_plugin = 'sha256_password'
+        user = self.users[auth_plugin]
+        self._create_user(self.admin_cnx, user['username'],
+                          user['password'],
+                          self.host,
+                          config['database'],
+                          plugin=auth_plugin)
+
+        config['user'] = user['username']
+        config['password'] = user['password']
+        config['client_flags'] = [constants.ClientFlag.PLUGIN_AUTH]
+        self.assertRaises(errors.InterfaceError, connection.MySQLConnection,
+                          **config)
+
+    @unittest.skipIf(tests.MYSQL_VERSION < (5, 5, 7),
+                     "MySQL {0} does not support authentication plugins".format(
+                        tests.MYSQL_VERSION_TXT))
+    def test_native(self):
+        config = tests.get_mysql_config()
+        config['unix_socket'] = None
+
+        auth_plugin = 'mysql_native_password'
+        user = self.users[auth_plugin]
+        self._create_user(self.admin_cnx, user['username'],
+                          user['password'],
+                          self.host,
+                          config['database'],
+                          plugin=auth_plugin)
+
+        config['user'] = user['username']
+        config['password'] = user['password']
+        config['client_flags'] = [constants.ClientFlag.PLUGIN_AUTH]
+        try:
+            cnx = connection.MySQLConnection(**config)
+        except Exception as exc:
+            self.fail("Connecting using {0} auth failed: {1}".format(
+                auth_plugin, exc))
+
+
+class BugOra18144971(tests.MySQLConnectorTests):
+    """BUG#18144971 ERROR WHEN USING UNICODE ARGUMENTS IN PREPARED STATEMENT"""
+    def setUp(self):
+        config = tests.get_mysql_config()
+        config['use_unicode'] = True
+        self.cnx = connection.MySQLConnection(**config)
+        self.cursor = self.cnx.cursor()
+
+        self.table = 'Bug18144971'
+        self.cursor.execute("DROP TABLE IF EXISTS {0}".format(self.table))
+
+        create = ("CREATE TABLE {0} ( "
+                  "`id` int(11) NOT NULL, "
+                  "`name` varchar(40) NOT NULL , "
+                  "`phone` varchar(40), "
+                  "PRIMARY KEY (`id`))"
+                  " CHARACTER SET 'utf8'".format(self.table))
+
+        self.cursor.execute(create)
+        self.table_cp1251 = 'Bug18144971_cp1251'
+        self.cursor.execute(
+            "DROP TABLE IF EXISTS {0}" .format(self.table_cp1251)
+        )
+
+        create = ("CREATE TABLE {0} ( "
+                  "`id` int(11) NOT NULL, "
+                  "`name` varchar(40) NOT NULL , "
+                  "`phone` varchar(40), "
+                  "PRIMARY KEY (`id`))"
+                  " CHARACTER SET 'cp1251'".format(self.table_cp1251))
+        self.cursor.execute(create)
+
+    def test_prepared_statement(self):
+        self.cur = self.cnx.cursor(prepared=True)
+        stmt = "INSERT INTO {0} VALUES (?,?,?)".format(
+               self.table)
+        data = BugOra18144971Extra.data
+        exp = BugOra18144971Extra.exp
+        self.cur.execute(stmt, data[0])
+        self.cnx.commit()
+        self.cur.execute("SELECT * FROM {0}".format(self.table))
+        self.assertEqual(self.cur.fetchall(), [exp[0]])
+
+        config = tests.get_mysql_config()
+        config['charset'] = 'cp1251'
+        self.cnx = connection.MySQLConnection(**config)
+        self.cur = self.cnx.cursor(prepared=True)
+        stmt = "INSERT INTO {0} VALUES (?,?,?)".format(
+               self.table_cp1251)
+        self.cur.execute(stmt, data[1])
+        self.cnx.commit()
+        self.cur.execute("SELECT * FROM {0}".format(self.table_cp1251))
+        self.assertEqual(self.cur.fetchall(), [exp[1]])
+
+
+class BugOra18389196(tests.MySQLConnectorTests):
+    """BUG#18389196:  INSERTING PARAMETER MULTIPLE TIMES IN STATEMENT
+    """
+    def setUp(self):
+        config = tests.get_mysql_config()
+        self.cnx = connection.MySQLConnection(**config)
+        self.cursor = self.cnx.cursor()
+
+        self.tbl = 'Bug18389196'
+        self.cursor.execute("DROP TABLE IF EXISTS %s" % self.tbl)
+
+        create = ("CREATE TABLE %s ( "
+                  "`id` int(11) NOT NULL, "
+                  "`col1` varchar(20) NOT NULL, "
+                  "`col2` varchar(20) NOT NULL, "
+                  "PRIMARY KEY (`id`))" % self.tbl)
+        self.cursor.execute(create)
+
+    def tearDown(self):
+        self.cursor.execute("DROP TABLE IF EXISTS %s" % self.tbl)
+        self.cursor.close()
+        self.cnx.close()
+
+    def test_parameters(self):
+        stmt = ("INSERT INTO {0} (id,col1,col2) VALUES "
+                "(%(id)s,%(name)s,%(name)s)".format(
+                self.tbl))
+        try:
+            self.cursor.execute(stmt, {'id': 1, 'name': 'ABC'})
+        except errors.ProgrammingError as err:
+            self.fail("Inserting parameter multiple times in a statement "
+                      "failed: %s" %
+                      err)
+
+
+class BugOra18415927(tests.MySQLConnectorTests):
+
+    """BUG#18415927: AUTH_RESPONSE VARIABLE INCREMENTED WITHOUT BEING DEFINED
+    """
+
+    user = {
+        'username': 'nativeuser',
+        'password': 'nativeP@ss',
+    }
+
+    def setUp(self):
+        config = tests.get_mysql_config()
+        host = config['host']
+        database = config['database']
+        cnx = connection.MySQLConnection(**config)
+        try:
+            cnx.cmd_query("DROP USER '{user}'@'{host}'".format(
+                host=host,
+                user=self.user['username']))
+        except:
+            pass
+
+        create_user = "CREATE USER '{user}'@'{host}' "
+        cnx.cmd_query(create_user.format(user=self.user['username'],
+                                         host=host))
+
+        passwd = ("SET PASSWORD FOR '{user}'@'{host}' = "
+                  "PASSWORD('{password}')").format(
+                      user=self.user['username'], host=host,
+                      password=self.user['password'])
+
+        cnx.cmd_query(passwd)
+
+        grant = "GRANT ALL ON {database}.* TO '{user}'@'{host}'"
+        cnx.cmd_query(grant.format(database=database,
+                                   user=self.user['username'],
+                                   host=host))
+
+    def tearDown(self):
+        config = tests.get_mysql_config()
+        host = config['host']
+        cnx = connection.MySQLConnection(**config)
+        cnx.cmd_query("DROP USER '{user}'@'{host}'".format(
+            host=host,
+            user=self.user['username']))
+
+    def test_auth_response(self):
+        config = tests.get_mysql_config()
+        config['unix_socket'] = None
+        config['user'] = self.user['username']
+        config['password'] = self.user['password']
+        config['client_flags'] = [-constants.ClientFlag.SECURE_CONNECTION]
+        try:
+            cnx = connection.MySQLConnection(**config)
+        except Exception as exc:
+            self.fail("Connection failed: {0}".format(exc))
+
+
+class BugOra18527437(tests.MySQLConnectorTests):
+    """BUG#18527437: UNITTESTS FAILING WHEN --host=::1 IS PASSED AS ARGUMENT
+    """
+    def test_poolname(self):
+        config = tests.get_mysql_config()
+        config['host'] = '::1'
+        config['pool_size'] = 3
+
+        exp = '{0}_{1}_{2}_{3}'.format(config['host'], config['port'],
+                                       config['user'], config['database'])
+        self.assertEqual(exp, pooling.generate_pool_name(**config))
+
+    def test_custom_poolname(self):
+        cnxpool = pooling.MySQLConnectionPool(pool_name='ham:spam',
+                                              **tests.get_mysql_config())
+        self.assertEqual('ham:spam', cnxpool._pool_name)
+        cnxpool._remove_connections()
+
+
+class BugOra18742429(tests.MySQLConnectorTests):
+    """BUG#18742429:  CPY FAILS WHEN QUERYING LARGE NUMBER OF COLUMNS
+    """
+    def setUp(self):
+        config = tests.get_mysql_config()
+        self.cnx = connection.MySQLConnection(**config)
+        self.cursor = self.cnx.cursor()
+
+        self.tbl = 'Bug18742429'
+        self.cursor.execute("DROP TABLE IF EXISTS %s" % self.tbl)
+
+        create = 'CREATE TABLE {0}({1})'.format(self.tbl, ','.join(
+            ['col'+str(i)+' INT(10)' for i in range(1000)]))
+
+        self.cursor.execute(create)
+
+    def tearDown(self):
+        self.cursor.execute("DROP TABLE IF EXISTS %s" % self.tbl)
+        self.cursor.close()
+        self.cnx.close()
+
+    def test_columns(self):
+        stmt = "INSERT INTO {0} VALUES({1})".format(self.tbl, ','.join(
+            [str(i) if i%2==0 else 'NULL' for i in range(1000)]
+        ))
+        exp = tuple(i if i%2==0 else None for i in range(1000))
+        self.cursor.execute(stmt)
+
+        self.cursor = self.cnx.cursor(prepared=True)
+        stmt = 'SELECT * FROM {0} WHERE col0=?'.format(self.tbl)
+        self.cursor.execute(stmt, (0,))
+        self.assertEqual(exp, self.cursor.fetchone())
+
+
+class BugOra18694096(tests.MySQLConnectorTests):
+    """BUG#18694096: Fixed negative conversion timedelta values
+    """
+
+    cases = [
+        (timedelta(hours=0, minutes=0, seconds=0, microseconds=1),
+         '00:00:00.000001'),
+        (timedelta(hours=0, minutes=0, seconds=0, microseconds=-1),
+         '-00:00:00.000001'),
+        (timedelta(days=2, hours=0, microseconds=1),
+         '48:00:00.000001'),
+        (timedelta(days=-3, seconds=86399, microseconds=999999),
+         '-48:00:00.000001'),
+        (timedelta(hours=0, minutes=0, seconds=1, microseconds=0),
+         '00:00:01',),
+        (timedelta(hours=0, minutes=0, seconds=-1, microseconds=0),
+         '-00:00:01'),
+        (timedelta(hours=0, minutes=1, seconds=1, microseconds=0),
+         '00:01:01'),
+        (timedelta(hours=0, minutes=-1, seconds=-1, microseconds=0),
+         '-00:01:01'),
+        (timedelta(hours=1, minutes=1, seconds=1, microseconds=0),
+         '01:01:01'),
+        (timedelta(hours=-1, minutes=-1, seconds=-1, microseconds=0),
+         '-01:01:01'),
+    ]
+
+    def setUp(self):
+        config = tests.get_mysql_config()
+        self.cnx = mysql.connector.connect(**config)
+
+        self.tbl = 'times'
+        self.cnx.cmd_query("DROP TABLE IF EXISTS {0}".format(self.tbl))
+        create = "CREATE TABLE {0} (c1 TIME(6))".format(self.tbl)
+        self.cnx.cmd_query(create)
+
+    def tearDown(self):
+        if self.cnx:
+            self.cnx.cmd_query("DROP TABLE IF EXISTS {0}".format(self.tbl))
+
+    def test_timedelta(self):
+        # Note that both _timedelta_to_mysql and _TIME_to_python are
+        # tested
+        cur = self.cnx.cursor()
+
+        # Following uses _timedelta_to_mysql to insert data
+        data = [(case[0],) for case in self.cases]
+        cur.executemany("INSERT INTO {0} (c1) VALUES (%s)".format(self.tbl),
+                         data)
+
+        # We use _TIME_to_python to convert back to Python
+        cur.execute("SELECT c1 FROM {0}".format(self.tbl))
+        for i, row in enumerate(cur.fetchall()):
+            self.assertEqual(self.cases[i][0], row[0],
+                             "Incorrect timedelta for {0}".format(
+                                 self.cases[i][1]))
+
+
+class BugOra19164627(tests.MySQLConnectorTests):
+    """BUG#19164627: Cursor tries to decode LINESTRING data as utf-8
+    """
+    def test_linestring(self):
+        config = tests.get_mysql_config()
+        cnx = mysql.connector.connect(**config)
+        cur = cnx.cursor()
+
+        cur.execute('DROP TABLE IF EXISTS BugOra19164627')
+        cur.execute("CREATE TABLE BugOra19164627 ( "
+                    "id SERIAL PRIMARY KEY AUTO_INCREMENT NOT NULL, "
+                    "line LINESTRING NOT NULL "
+                    ") DEFAULT CHARSET=ascii")
+        cur.execute('INSERT IGNORE INTO BugOra19164627(id, line) '
+                    'VALUES (0,LINESTRING(POINT(0, 0), POINT(0, 1)))')
+
+        cur.execute("SELECT * FROM BugOra19164627 LIMIT 1")
+        self.assertEqual(cur.fetchone(), (1, b'\x00\x00\x00\x00\x01\x02\x00\x00'
+                                             b'\x00\x02\x00\x00\x00\x00\x00\x00'
+                                             b'\x00\x00\x00\x00\x00\x00\x00\x00'
+                                             b'\x00\x00\x00\x00\x00\x00\x00\x00'
+                                             b'\x00\x00\x00\x00\x00\x00\x00\x00'
+                                             b'\x00\x00\x00\xf0?', ))
+        cur.execute('DROP TABLE IF EXISTS BugOra19164627')
+        cur.close()
+        cnx.close()
+
+
+class BugOra19225481(tests.MySQLConnectorTests):
+    """BUG#19225481: FLOATING POINT INACCURACY WITH PYTHON v2
+    """
+    def setUp(self):
+        config = tests.get_mysql_config()
+        self.cnx = connection.MySQLConnection(**config)
+        self.cursor = self.cnx.cursor()
+
+        self.tbl = 'Bug19225481'
+        self.cursor.execute("DROP TABLE IF EXISTS %s" % self.tbl)
+
+        create = 'CREATE TABLE {0}(col1 DOUBLE)'.format(
+            self.tbl)
+
+        self.cursor.execute(create)
+
+    def tearDown(self):
+        self.cursor.execute("DROP TABLE IF EXISTS %s" % self.tbl)
+        self.cursor.close()
+        self.cnx.close()
+
+    def test_columns(self):
+        values = [
+            (123.123456789987,),
+            (234.234,),
+            (12.12,),
+            (111.331,),
+            (0.0,),
+            (-99.99999900099,)
+        ]
+        stmt = "INSERT INTO {0} VALUES(%s)".format(self.tbl)
+        self.cursor.executemany(stmt, values)
+
+        stmt = "SELECT * FROM {0}".format(self.tbl)
+        self.cursor.execute(stmt)
+        self.assertEqual(values, self.cursor.fetchall())
